@@ -55,6 +55,7 @@ QT_TRANSLATE_NOOP("JMdictLongDescs", "word containing out-dated kanji");
 QT_TRANSLATE_NOOP("JMdictLongDescs", "obsolete term");
 QT_TRANSLATE_NOOP("JMdictLongDescs", "obscure term");
 QT_TRANSLATE_NOOP("JMdictLongDescs", "out-dated or obsolete kana usage");
+QT_TRANSLATE_NOOP("JMdictLongDescs", "old or irregular kana form");
 QT_TRANSLATE_NOOP("JMdictLongDescs", "onomatopoeic or mimetic word");
 QT_TRANSLATE_NOOP("JMdictLongDescs", "pronoun");
 QT_TRANSLATE_NOOP("JMdictLongDescs", "poetical term");
@@ -146,6 +147,7 @@ QT_TRANSLATE_NOOP("JMdictLongDescs", "Nidan verb (lower class) with `yu' ending
 QT_TRANSLATE_NOOP("JMdictLongDescs", "Nidan verb (lower class) with `ru' ending (archaic)");
 QT_TRANSLATE_NOOP("JMdictLongDescs", "Nidan verb (lower class) with `u' ending and `we' conjugation (archaic)");
 QT_TRANSLATE_NOOP("JMdictLongDescs", "architecture term");
+QT_TRANSLATE_NOOP("JMdictLongDescs", "anatomical term");
 QT_TRANSLATE_NOOP("JMdictLongDescs", "astronomy, etc. term");
 QT_TRANSLATE_NOOP("JMdictLongDescs", "baseball term");
 QT_TRANSLATE_NOOP("JMdictLongDescs", "biology term");
