@@ -1,18 +1,18 @@
 /*
- *	Copyright (C) 2010	Alexandre Courbot
+ * Copyright (C) 2010 Alexandre Courbot
  *
- *	This program is free software: you can redistribute it and/or modify
- *	it under the terms of the GNU General Public License as published by
- *	the Free Software Foundation, either version 3 of the License, or
- *	(at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *	This program is distributed in the hope that it will be useful,
- *	but WITHOUT ANY WARRANTY; without even the implied warranty of
- *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *	GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *	You should have received a copy of the GNU General Public License
- *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "sqlite/Connection.h"
@@ -29,11 +29,11 @@
 
 #include <QtDebug>
 
-#define BIND(query, val) { if (!query.bindValue(val)) { qFatal("%s", query.lastError().message().toUtf8().data()); return false; } }
-#define BINDNULL(query) { if (!query.bindNullValue()) { qFatal("%s", query.lastError().message().toUtf8().data()); return false; } }
+#define BIND(query, val) { if (!query.bindValue(val)) { qCritical("%s", query.lastError().message().toUtf8().data()); return false; } }
+#define BINDNULL(query) { if (!query.bindNullValue()) { qCritical("%s", query.lastError().message().toUtf8().data()); return false; } }
 #define AUTO_BIND(query, val, nval) if (val == nval) BINDNULL(query) else BIND(query, val)
-#define EXEC(query) if (!query.exec()) { qFatal("%s", query.lastError().message().toUtf8().data()); return false; }
-#define EXEC_STMT(query, stmt) if (!query.exec(stmt)) { qFatal("%s", query.lastError().message().toUtf8().data()); return false; }
+#define EXEC(query) if (!query.exec()) { qCritical("%s", query.lastError().message().toUtf8().data()); return false; }
+#define EXEC_STMT(query, stmt) if (!query.exec(stmt)) { qCritical("%s", query.lastError().message().toUtf8().data()); return false; }
 #define ASSERT(cond) { if (!cond) { qCritical("%s: assert condition failed, line %d", __FILE__, __LINE__); return false; } }
 
 class JMdictDBParser : public JMdictParser
@@ -58,7 +58,9 @@ public:
 	bool finalizeLanguagesDatabases();
 	bool prepareLanguagesQueries();
 	bool clearLanguagesQueries();
-	bool fillLanguagesInfoTable();	
+	bool fillLanguagesInfoTable();
+	bool parseJMFs(const QStringList &supportedLanguages);
+	bool parseJMF(const QString &fname, const QString &lang);
 	bool insertJLPTLevel(const QString& fName, int level);
 	bool insertJLPTLevels();
 	bool populateEntitiesTable();
@@ -77,6 +79,8 @@ private:
 	QMap<QString, SQLite::Query> insertGlossTextQueries;
 	QMap<QString, SQLite::Query> insertGlossQueries;
 	QMap<QString, SQLite::Query> insertGlossesQueries;
+	// lang ; id ; pri ; str
+	QMap<QString, QMap<int, QMap<int, QStringList> > > jmf;
 	
 	bool openDatabase(QString databaseName, QString handle);
 	bool closeDatabase(QString handle);
@@ -152,25 +156,27 @@ bool JMdictDBParser::onItemParsed(const JMdictItem &entry)
 		foreach (quint8 res, sense.restrictedToKana) restrictedToList << QString::number(res);
 		AUTO_BIND(insertSenseQuery, restrictedToList.join(","), "");
 		EXEC(insertSenseQuery);
-		
-		// Insert the gloss for all languages
+
+
+		// Insert the gloss for all languages (search table)
 		foreach (const QString &lang, languages) {
-			if (!sense.gloss.contains(lang)) {
+			// Do we have a replacement for the glosses?
+			const QStringList &glosses = jmf[lang][entry.id][idx].isEmpty() ?
+				sense.gloss[lang] : jmf[lang][entry.id][idx];
+			if (glosses.isEmpty()) {
 				allGlosses[lang] << QString();
 				continue;
 			}
-			allGlosses[lang] << sense.gloss[lang].join("\n");
-			BIND(insertGlossTextQueries[lang], sense.gloss[lang].join(", "));
+			allGlosses[lang] << glosses.join("\n");
+			BIND(insertGlossTextQueries[lang], glosses.join(", "));
 			EXEC(insertGlossTextQueries[lang]);
-			qint64 rowId = insertGlossTextQueries[lang].lastInsertId();
 			BIND(insertGlossQueries[lang], entry.id);
-			BIND(insertGlossQueries[lang], rowId);
+			BIND(insertGlossQueries[lang], insertGlossTextQueries[lang].lastInsertId());
 			EXEC(insertGlossQueries[lang]);
 		}
 		++idx;
 	}
-	// Insert the glosses for every language
-	// TODO do not insert for non-present glosses
+	// For every language, insert all glosses of an entry (load table)
 	foreach (const QString &lang, languages) {
 		QString all(allGlosses[lang].join("\n\n"));
 		if (all.split("\n", QString::SkipEmptyParts).empty()) continue;
@@ -218,10 +224,11 @@ bool JMdictDBParser::insertJLPTLevel(const QString &fName, int level)
 
 bool JMdictDBParser::insertJLPTLevels()
 {
-	ASSERT(insertJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/jmdict/jlpt-level4.txt"), 4));
-	ASSERT(insertJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/jmdict/jlpt-level3.txt"), 3));
-	ASSERT(insertJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/jmdict/jlpt-level2.txt"), 2));
-	ASSERT(insertJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/jmdict/jlpt-level1.txt"), 1));
+	ASSERT(insertJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/jmdict/jlpt-n1.csv"), 1));
+	ASSERT(insertJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/jmdict/jlpt-n2.csv"), 2));
+	ASSERT(insertJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/jmdict/jlpt-n3.csv"), 3));
+	ASSERT(insertJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/jmdict/jlpt-n4.csv"), 4));
+	ASSERT(insertJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/jmdict/jlpt-n5.csv"), 5));
 	return true;	
 }
 
@@ -235,7 +242,7 @@ bool JMdictDBParser::openDatabase(QString databaseName, QString handle)
 		return false;
 	}
 	if (!connection.connect(dbFile, SQLite::Connection::JournalInFile)) {
-		qFatal("Cannot open database: %s", connection.lastError().message().toLatin1().data());
+		qCritical("Cannot open database: %s", connection.lastError().message().toLatin1().data());
 		return false;
 	}
 	connection.transaction();
@@ -448,6 +455,42 @@ bool JMdictDBParser::populateEntitiesTable()
 	return true;
 }
 
+bool JMdictDBParser::parseJMF(const QString &fName, const QString &lang)
+{
+	QFile file(fName);
+	if (!file.open(QFile::ReadOnly | QFile::Text)) {
+		return false;
+	}
+
+	QTextStream in(&file);
+	QString line(in.readLine());
+	while (!line.isNull()) {
+		int pos = line.indexOf(' ');
+		int pos2 = line.indexOf(' ', pos + 1);
+		int eid(line.left(pos).toInt());
+		int pri(line.mid(pos + 1, pos2 - (pos + 1)).toInt());
+
+		// Flush previous 
+		QString gloss(line.mid(pos2 + 1));
+		jmf[lang][eid][pri] << gloss;
+		line = in.readLine();
+	}
+	return true;
+}
+
+bool JMdictDBParser::parseJMFs(const QStringList &supportedLanguages)
+{
+	QDir dir(QDir(srcDir).absoluteFilePath("src/core/jmdict"));
+
+	foreach (QString fName, dir.entryList(QStringList() << "*.jmf")) {
+		QString lang(fName.split('.')[0]);
+		if (supportedLanguages.contains(lang))
+			parseJMF(dir.absoluteFilePath(fName), lang);
+	}
+
+	return true;
+}
+
 void printUsage(char *argv[])
 {
 	qCritical("Usage: %s [-l<lang>] source_dir dest_dir\nWhere <lang> is a two-letters language code (en, fr, de, es or ru)", argv[0]);
@@ -457,6 +500,7 @@ bool buildDB(const QStringList &languages, const QString &srcDir, const QString
 {
 	JMdictDBParser parser(languages, srcDir, dstDir);
 
+	parser.parseJMFs(languages);
 	parser.createMainDatabase();
 	parser.createMainTables();
 	parser.prepareMainQueries();
@@ -492,7 +536,7 @@ bool buildDB(const QStringList &languages, const QString &srcDir, const QString
 int main(int argc, char *argv[])
 {
 	QCoreApplication app(argc, argv);
-	SQLite::init_sqlite_extensions();
+	sqlite3ext_init();
 	
 	if (argc < 3) { 
 		printUsage(argv); return 1;
