@@ -1,18 +1,18 @@
 /*
- *	Copyright (C) 2010	Alexandre Courbot
+ * Copyright (C) 2010 Alexandre Courbot
  *
- *	This program is free software: you can redistribute it and/or modify
- *	it under the terms of the GNU General Public License as published by
- *	the Free Software Foundation, either version 3 of the License, or
- *	(at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *	This program is distributed in the hope that it will be useful,
- *	but WITHOUT ANY WARRANTY; without even the implied warranty of
- *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *	GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *	You should have received a copy of the GNU General Public License
- *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "sqlite/Connection.h"
@@ -28,11 +28,11 @@
 
 #include <QtDebug>
 
-#define BIND(query, val) { if (!query.bindValue(val)) { qFatal("%s", query.lastError().message().toUtf8().data()); return false; } }
-#define BINDNULL(query) { if (!query.bindNullValue()) { qFatal("%s", query.lastError().message().toUtf8().data()); return false; } }
+#define BIND(query, val) { if (!query.bindValue(val)) { qCritical("%s", query.lastError().message().toUtf8().data()); return false; } }
+#define BINDNULL(query) { if (!query.bindNullValue()) { qCritical("%s", query.lastError().message().toUtf8().data()); return false; } }
 #define AUTO_BIND(query, val, nval) { if (val == nval) BINDNULL(query) else BIND(query, val) }
-#define EXEC(query) { if (!query.exec()) { qFatal("%s", query.lastError().message().toUtf8().data()); return false; } }
-#define EXEC_STMT(query, stmt)  { if (!query.exec(stmt)) { qFatal("%s", query.lastError().message().toUtf8().data()); return false; } }
+#define EXEC(query) { if (!query.exec()) { qCritical("%s", query.lastError().message().toUtf8().data()); return false; } }
+#define EXEC_STMT(query, stmt)  { if (!query.exec(stmt)) { qCritical("%s", query.lastError().message().toUtf8().data()); return false; } }
 #define ASSERT(cond) { if (!cond) { qCritical("%s: assert condition failed, line %d", __FILE__, __LINE__); return false; } }
 
 QMap<quint32, quint8> knownRadicals;
@@ -68,6 +68,7 @@ public:
 	virtual bool onItemParsed(Kanjidic2Item &kanji);
 	bool updateJLPTLevel(const QString &fName, int level);
 	bool updateJLPTLevels();
+
 private:
 };
 
@@ -78,7 +79,7 @@ bool Kanjidic2DBParser::onItemParsed(Kanjidic2Item &kanji)
 	AUTO_BIND(insertEntryQuery, kanji.grade, 0);
 	AUTO_BIND(insertEntryQuery, kanji.stroke_count, 0);
 	AUTO_BIND(insertEntryQuery, (kanji.freq == 0 ? 0 : 2502 - kanji.freq), 0);
-	AUTO_BIND(insertEntryQuery, kanji.jlpt, 0);
+	AUTO_BIND(insertEntryQuery, (kanji.jlpt >= 3 ? kanji.jlpt + 1 : kanji.jlpt), 0);
 	AUTO_BIND(insertEntryQuery, kanji.heisig, 0);
 	EXEC(insertEntryQuery);
 	
@@ -171,7 +172,7 @@ bool Kanjidic2DBParser::updateJLPTLevel(const QString &fName, int level)
 		return false;
 	};
 	QTextStream in(&file);
-	QString line = in.readLine();
+	QString line(in.readLine());
 	while (!line.isNull()) {
 		BIND(updateJLPTLevelsQuery, level);
 		BIND(updateJLPTLevelsQuery, TextTools::singleCharToUnicode(line));
@@ -183,9 +184,11 @@ bool Kanjidic2DBParser::updateJLPTLevel(const QString &fName, int level)
 
 bool Kanjidic2DBParser::updateJLPTLevels()
 {
-	ASSERT(updateJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/kanjidic2/jlpt-level2.txt"), 2));
-	ASSERT(updateJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/kanjidic2/jlpt-level3.txt"), 3));
-	ASSERT(updateJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/kanjidic2/jlpt-level4.txt"), 4));
+	ASSERT(updateJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/kanjidic2/jlpt-n1.csv"), 1));
+	ASSERT(updateJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/kanjidic2/jlpt-n2.csv"), 2));
+	ASSERT(updateJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/kanjidic2/jlpt-n3.csv"), 3));
+	ASSERT(updateJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/kanjidic2/jlpt-n4.csv"), 4));
+	ASSERT(updateJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/kanjidic2/jlpt-n5.csv"), 5));
 	return true;
 }
 
@@ -287,6 +290,10 @@ public:
 	bool parse();
 	bool fillMainInfoTable();
 	bool fillLanguagesInfoTable();
+
+	bool updateTranslations(const QStringList &supportedLanguages);
+	bool updateTranslation(const QString &fName, const QString &lang);
+
 private:
 	QStringList languages;
 	KanjiVGDBParser kvgParser;
@@ -365,7 +372,7 @@ bool KanjiDB::openDatabase(QString databaseName, QString handle)
 		return false;
 	}
 	if (!connection.connect(dbFile, SQLite::Connection::JournalInFile)) {
-		qFatal("Cannot open database: %s", connection.lastError().message().toLatin1().data());
+		qCritical("Cannot open database: %s", connection.lastError().message().toLatin1().data());
 		return false;
 	}
 	connection.transaction();
@@ -387,9 +394,9 @@ bool KanjiDB::createRootComponentsTable()
 {
 	SQLite::Query query(&connections["main"]);
 	if (!query.exec("select distinct ks.element from strokeGroups as ks join entries as e on ks.element = e.id where ks.element not in (select distinct kanji from strokeGroups where element != kanji) "
-	                // We are not counting components that are only components of themselves (whatever that means)
-	                "and ks.kanji != ks.element "
-	                "order by strokeCount")) {
+	        // We are not counting components that are only components of themselves (whatever that means)
+	        "and ks.kanji != ks.element "
+	        "order by strokeCount")) {
 		return false;
 	}
 	while (query.next()) {
@@ -509,6 +516,62 @@ bool KanjiDB::fillLanguagesInfoTable()
 	return true;
 }
 
+bool KanjiDB::updateTranslations(const QStringList &supportedLanguages)
+{
+	QDir dir(QDir(srcDir).absoluteFilePath("src/core/kanjidic2"));
+
+	foreach (QString fName, dir.entryList(QStringList() << "*.jmf")) {
+		QString lang(fName.split('.')[0]);
+		if (supportedLanguages.contains(lang))
+			updateTranslation(dir.absoluteFilePath(fName), lang);
+	}
+
+	return true;
+}
+
+bool KanjiDB::updateTranslation(const QString &fName, const QString &lang)
+{
+	QFile file(fName);
+	if (!file.open(QFile::ReadOnly | QFile::Text)) {
+		return false;
+	}
+
+	SQLite::Query deleteMeaningQuery(&connections[lang]);
+	SQLite::Query deleteMeaningTextQuery(&connections[lang]);
+	deleteMeaningTextQuery.prepare("delete from meaningText where docid in (select docid from meaning where entry = ?)");
+	deleteMeaningQuery.prepare("delete from meaning where entry = ?");
+	SQLite::Query &mQuery = insertMeaningQueries[lang];
+	SQLite::Query &mtQuery = insertMeaningTextQueries[lang];
+	QTextStream in(&file);
+
+	int lastKanji = 0;
+	QString line(in.readLine());
+	while (!line.isNull()) {
+		int pos = line.indexOf(' ');
+		int kanji(TextTools::singleCharToUnicode(line.left(pos)));
+		QString meaning(line.mid(pos + 1));
+
+		if (lastKanji != kanji) {
+			BIND(deleteMeaningTextQuery, kanji);
+			EXEC(deleteMeaningTextQuery);
+			BIND(deleteMeaningQuery, kanji);
+			EXEC(deleteMeaningQuery);
+			lastKanji = kanji;
+		}
+
+		// TODO factorize identical meanings! Record the row id into a hash table
+		BIND(mtQuery, meaning);
+		EXEC(mtQuery);
+		BIND(mQuery, mtQuery.lastInsertId());
+		BIND(mQuery, kanji);
+		BIND(mQuery, qCompress(meaning.toUtf8(), 9));
+		EXEC(mQuery);
+
+		line = in.readLine();
+	}
+	return true;
+}
+
 bool KanjiDB::parse()
 {
 	// Parse and insert kanjidic2
@@ -556,6 +619,7 @@ bool buildDB(const QStringList &languages, const QString &srcDir, const QString
 	ASSERT(kanjiDB.createTables());
 	ASSERT(kanjiDB.prepareQueries());
 	ASSERT(kanjiDB.parse());
+	ASSERT(kanjiDB.updateTranslations(languages));
 	ASSERT(kanjiDB.createIndexes());
 	ASSERT(kanjiDB.clearQueries());
 	ASSERT(kanjiDB.finalize());
